# --
# Kernel/System/ProductLicense.pm - core module for product license management
# Copyright (C) 2006-2014 c.a.p.e. IT GmbH, http://www.cape-it.de
#
# written/edited by:
# * Rene(dot)Boehm(at)cape(dash)it(dot)de
#
# --
# This software comes with ABSOLUTELY NO WARRANTY. For details, see
# the enclosed file COPYING for license information (AGPL). If you
# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
# --

package Kernel::System::ProductLicense;

#use strict;
use warnings;

use MIME::Base64;

use Kernel::System::Main;
use Kernel::System::Time;
use Kernel::System::User;
use Kernel::System::SysConfig;

our $CORE = <<COREEND;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COREEND

sub new {
    my ( $Type, %Param ) = @_;

    # allocate new hash for object
    my $Self = {%Param};
    bless( $Self, $Type );

    # check needed objects
    for my $Object (qw(LogObject ConfigObject DBObject)) {
        $Self->{$Object} = $Param{$Object} || die "Got no $Object!";
    }
           
    return $Self;
}

sub Do {
    my ( $Self, %Param ) = @_;
    
    if (!$Self->{Decoded}) {
        my $Raw = decode_base64($CORE);
        eval($Raw);
        $Self->{Decoded} = 1;
    }
        
    my $Function = $Param{Function};
    return &$Function($Self, %Param);
}

1;
